#!/usr/bin/env node
(function() {
  var ACTIONS, HELP, fs, main;
  fs = require('fs');
  HELP = {
    main: function(argv) {
      var action, k, v, _results;
      if (argv.length === 0) {
        console.log("Specify one of these actions :");
        _results = [];
        for (k in ACTIONS) {
          v = ACTIONS[k];
          _results.push(console.log(" - ", k, " : ", v.DESCRIPTION));
        }
        return _results;
      } else if (argv.length === 1) {
        action = argv[0];
        if (action in ACTIONS) {
          return ACTIONS[action].help();
        } else {
          throw "Command does not exists : " + action;
        }
      } else {
        throw "Invalid command.";
      }
    }
  };
  ACTIONS = {
    serve: require("./tumbler-server.js"),
    deploy: require("./tumbler-deploy.js"),
    build: require("./tumbler-build.js"),
    help: HELP
  };
  main = function(argv) {
    var action;
    try {
      if (argv.length > 0) {
        action = argv[0];
        if (ACTIONS[action] != null) {
          return ACTIONS[action].main(argv.slice(1));
        } else {
          throw '"' + action + '" is not an allowed action.';
        }
      } else {
        throw "You need to specify an action.";
      }
    } catch (e) {
      console.log(e);
      console.log(e.stack);
      return HELP.main([]);
    }
  };
  exports.main = main;
  if (require.main === module) {
    main(process.argv.slice(2));
  }
}).call(this);
